# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template1LevNormOrt(Template):
    'Fits 1-level Normal models with the option to reparameterise explanatory variables into an orthogonal list to improve MCMC mixing.'

    __version__ = '1.0.0'

    tags = [ 'Model', '1-Level', 'Normal', 'Alternative MCMC methods', 'Orthogonal parameterisation' ]
    engines = ['eStat', 'MLwiN_MCMC']

    inputs = '''
y = DataVector('Response: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)
useorthog = Boolean('Do you want to use orthogonal parameterisation?: ')
if useorthog:
    orthtype = Text('Type:', ['Orthogonal', 'Orthonormal'])

tau = ParamScalar()
sigma2 = ParamScalar(modelled = False)
origx = DataMatrix(value=x.name[:])
if useorthog:
    betaort = ParamVector(parents=[x], as_scalar=True)
    orthogmat = List(value = [])
    beta = ParamVector(parents=[x], as_scalar=True, modelled = False)
else:
    beta = ParamVector(parents=[x], as_scalar=True)
deviance = ParamScalar(modelled = False)
'''

    model = '''
model{
    for (i in 1:length(${y.name})) {
        ${y.name}[i] ~ dnorm(mu[i], tau)
        mu[i] <- \\
%if useorthog:
${mmult(x.name, 'betaort', 'i')}
% else:
${mmult(x.name, 'beta', 'i')}
% endif
    }
    
    # Priors
    % for i in range(0, x.ncols()):
%if useorthog:
    betaort_${i} ~ dflat()
% else:
    beta_${i} ~ dflat()
% endif
    % endfor

% if useorthog:
<% count = 0%>
    % for i in range(0, x.ncols()):
    beta_${i} <- \\
    % for j in range(0, x.ncols()):
${orthogmat[count]} * betaort_${j}\\
% if j == (x.ncols() - 1):

% else:
% if float(orthogmat[count+1])  >= float(0.0) :
 + \\
%endif
% endif
<% count += 1 %>\\
    % endfor
    % endfor
% endif

    tau ~ dgamma(0.001000, 0.001000)
    sigma2 <- 1 / tau
}
        '''

    latex = R'''
\begin{aligned}
 \mbox{${y.name}}_i & \sim \mbox{N}(\mu_i, \sigma^2) \\ 
\mu_i & =
%if useorthog:
        ${mmulttex(x.name, R'\beta^*', 'i')} \\ 
%for i in range(0, len(x.name)):
\beta^*_{${i}} & \propto 1 \\ 
%endfor
<% count = 0%>
    % for i in range(0, x.ncols()):
    \beta_{${i}} & = 
    % for j in range(0, x.ncols()):
${orthogmat[count]} \beta^*_${j}
% if j == (x.ncols() - 1):

% else:
% if float(orthogmat[count+1])  >= float(0.0) :
 + 
% endif
% endif
<% count += 1 %>
% endfor
\\ 
% endfor
% else:
        ${mmulttex(x.name, R'\beta', 'i')} 
%for i in range(0, len(x.name)):
\beta_{${i}} & \propto 1 \\ 
% endfor
% endif

\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau
\end{aligned} 
'''

    mlnscript = '''
RESP "${y}"
IDEN 1 "_id"
ADDT "_levres"
SETV 1 "_levres"
FPAR 0 "_levres"
% for i in range(len(origx)):
ADDT "${origx[i]}"
% endfor
'''

    preparedata = '''
from EStat.stats.utils.orthog import orthog
import numpy
mydata = data['datafile']

if useorthog:
    orth = numpy.zeros([len(mydata.variables[x[0]]['data']), len(x)])

    for i in range(0, len(x)):
        orth[:, i] = mydata.variables[x[i]]['data']

    if orthtype == 'Orthogonal':
        (tmp, om) = orthog(orth)

        orthogmat[:] = [str(i) for i in om.flat]

        for n in range(0, len(x)):
            mydata.addvariable('orth' + x[n], data = numpy.array(tmp[:, n]).flatten())

    if orthtype == 'Orthonormal':
        (tmp, om) = numpy.linalg.qr(numpy.mat(orth))

        orthogmat[:] = [str(i) for i in om.I.flat]

        for n in range(0, len(x)):
            mydata.addvariable('orth' + x[n], data = numpy.array(tmp[:, n]).flatten())

    x[:] = ['orth' + n for n in x]
'''
