# Copyright (c) 2013, University of Bristol and University of Southampton.
# Adapted from templates 2levelwithRS and 2levelComplex

from EStat.Templating import Template

class Template2LevRSComplex(Template):       
    'Fits 2-level Normal models, allowing for random slopes and heteroskedasticity in the response. Can specify either uniform or Wishart prior for the variance matrix.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal', 'Random slopes', 'Complex level 1', 'MLwiN:point & click']
    engines = ['eStat', 'MLwiN_MCMC']

    inputs = '''

def combs(s):
    out = []
    for i in range(0, len(s)):
        for j in range(0, i + 1):
            out.append(s[i] + '*' + s[j])
    return out

y = DataVector('Response: ')
L2ID = IDVector('Level 2 ID: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)
x2 = DataMatrix('Explanatory variables random at level 2: ', allow_cat = True)

complexvars = combs(x)
for i in range(0, len(complexvars)):
    context['inc' + str(i)] = Boolean('Include level 1 variation for ' + complexvars[i] + '? ')

storeresid = Boolean('Store level 2 residuals?') 

if storeresid:
    for var in range(0, len(x2)):
        context['u' + str(var)] = ParamVector(parents=[L2ID], as_scalar=False)
else:
    for var in range(0, len(x2)):
        context['u' + str(var)] = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)

beta = ParamVector(parents=[x], as_scalar=True)

num = len(x2)
if num == 1:
    tau_u = ParamScalar()
    sigma_u = ParamScalar(modelled = False)
else:
    omega_u = ParamMatrix(modelled = False, customstep=True)
    omega_u.size = num
    d_u = ParamMatrix(customstep=True)
    d_u.size = num 
    priors = Text('Priors: ', ['Uniform', 'Wishart'])
    if priors.name == 'Wishart':
        R = List('R matrix: ')
        v = Integer('Degrees of Freedom:')
for i in range(0, len(complexvars)):
    if context['inc' + str(i)]:
        context['alpha' + str(i)] = ParamScalar()

deviance = ParamScalar(modelled=False)  
'''

    model = '''
model {
    for (i in 1:length(${y})) {
        ${y}[i] ~ dnorm(mu[i], tau[i])
        mu[i] <- ${mmult(x, 'beta', 'i')} \\

            % for j in range(0, len(x2.name)):
              + u${j}[${L2ID.name}[i]] * ${x2[j]}[i] \\
            % endfor

            tau[i] <- 1/( \\
<% complexvars = combsi(x, 'i') %> \\
<% first = True %> \\
            % for i in range(0, len(complexvars)):
            % if context['inc' + str(i)]:
            % if first != True:
              + \\
            % endif
alpha${i} * ${complexvars[i]} \\
<% first = False %> \\
            % endif
            % endfor
)            
    }

<% num = len(x2.name) %> \\
    for(i1 in 1:length(u0)) {
        %if num == 1 :
           u0[i1] ~ dnorm(0, tau_u)
        %else:
           dummy[i1] ~ dnormal${num}a( \\
           % for j in range(0, num):
u${j}[i1], \\
           % endfor
           % for j in range(0, num):
0, \\
           % endfor
           % for j in range(0,(((num+1)*num)/2) - 1):
d_u[${j}], \\
           % endfor
d_u[${(((num+1)*num)/2) - 1}])
           dummy2[i1] ~ ddummy(dummy[i1])
           % for j in range(0, num):
           u${j}[i1] ~ dflat()
           % endfor

        %endif
    }
    
    # Priors
    % for i in range(0, len(x)):
    beta_${i} ~ dflat()
    % endfor
        
    %if len(x2.name) == 1 :
    tau_u ~ dgamma(0.001000, 0.001000)
    sigma_u <- 1 / sqrt(tau_u)
    % endif
    % for i in range(0, len(complexvars)):
    % if context['inc' + str(i)]:
    alpha${i} ~ dflat()
    % endif
    % endfor    
}
'''

    latex = r'''
<%!
def combslat(s, index):
        out = []
        for i in range(0, len(s)):
            for j in range(0, i + 1):
                if s[j]==s[i]:
                    out.append(r'\mbox{' + s[i] + '}_' + index +  '^2')
                else :
                    out.append(r'2 \mbox{'+s[i]+'}_'+index +r'\mbox{'+s[j]+'}_'+index)
        return out 
%>
\begin{aligned}
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma_i^2) \\ 
\mu_i & =
  ${mmulttex(x, r'\beta', 'i')}
<% num = len(x2.name) %>
        %if num == 1 :
+u_{\mbox{${L2ID}}[i]}\\
%else:
%for i in range(0, len(x2.name)):
+u_{${i}\mbox{${L2ID}[i]}}\mbox{${x2[i]}}_i
%endfor
%endif
<% num = len(x2.name) %> \\
        %if num == 1 :
\\ u_{\mbox{${L2ID}[i]}} & \sim \mbox{N}(0,\sigma^2_u)\\
%else:
\ \begin{bmatrix}
%for i in range(0, len(x2.name)):
u_{${i}\mbox{${L2ID}[i]}}\\ 
%endfor
\end{bmatrix}
&\sim\mbox{N}\left(
\begin{bmatrix}
%for i in range(0, len(x2.name)):
0\\ 
%endfor
\end{bmatrix}
,
\begin{bmatrix}
%for i in range(0, len(x2.name)):
%for j in range(0, len(x2.name)):
%if i==j:
\sigma^2_{u${i}}\\ 
%else:
%if i<j:
\sigma_{u${i}${j}}&
%endif
%endif
%endfor
%endfor
\end{bmatrix}
\right)\\ 
%endif
\sigma_i^2 & =  
   <% complexvars = combslat(x, 'i') %>      
 <% first = True %>
        % for i in range(0, len(complexvars)):
        % if context['inc' + str(i)]:
        % if first != True:
        +
        % endif
        ${complexvars[i]} \alpha_${i} 
        <% first = False %>
        % endif
        % endfor
\\ 
\beta_0 & \propto 1 \\ 
%for i in range(1, len(x)):
\beta_{${i}} & \propto 1 \\ 
%endfor
<% num = len(x2.name) %>
        %if num == 1 :
\tau_u & \sim \Gamma (0.001,0.001) \\ 
\sigma_u^2 & = 1 / \tau_u
        %else:
           \begin{bmatrix}
           %for i in range(0, len(x2.name)):
             %for j in range(0, len(x2.name)):
               %if i==j:
                  \sigma^2_{u${i}}\\ 
               %else:
                  %if i<j:
                     \sigma_{u${i}${j}}&
                  %endif
               %endif
             %endfor
           %endfor
           \end{bmatrix}
           &\propto
           %if priors.name == 'Uniform':
             \begin{bmatrix}
             %for i in range(0, len(x2.name)):
              %for j in range(0, len(x2.name)):
               %if i==j:
                  1\\ 
               %else:
                  %if i<j:
                     1&
                  %endif
               %endif
             %endfor
           %endfor
           \end{bmatrix}\\ 
           %else:
              %if priors.name == 'Wishart':
                \mbox{W}_{${len(x2.name)}}\left(
                \mbox{specified R matrix}
                ,
                ${v}\right)\\ 
              %endif
           %endif
        %endif
%for i in range(0, len(complexvars)):
    % if context['inc'+str(i)]:
    \alpha_{${i}} & \propto 1 \\ 
    % endif
%endfor
\end{aligned} 
'''
    
    preccode = '''
            {
<% n = len(x2) %>\\
% if n > 1 :
                // Note currently using a uniform prior for variance matrix
                RectMatrix sb(${n},${n});
                for(int i = 0; i < length(u0); i++) {
% for j in range(0, n):
% for k in range(0, n):
                    sb(${j}, ${k}) += double(u${j}[i]) * double(u${k}[i]);
% endfor
% endfor
                }
% if priors.name == 'Uniform':
                int vw = length(u0)-${n + 1};
                if (runstate == 0) {
% for j in range(0, n):
                    sb(${j}, ${j}) += 0.0001;
% endfor
                }
% endif
% if priors.name == 'Wishart':
                int vw = length(u0) + ${v.name};
% endif

% if priors.name == 'Wishart':
<%
import numpy
Rmat = numpy.empty([n, n])
count = 0 
for j in range(0, n):
    for k in range(0, j + 1):
        Rmat[j, k] = float(R.name[count]) 
        Rmat[k, j] = Rmat[j, k]
        count += 1
%>

<% count = 0 %>
% for j in range(0, n):
% for k in range(0, n):
                sb(${j}, ${k}) += ${str(Rmat[j, k] * float(v.name))};
% endfor
% endfor
% endif

                matrix_sym_invinplace(sb);
                bool fail = false;
                mat_d_u = dwishart(vw, sb, fail);
                mat_omega_u = matrix_sym_inverse(mat_d_u, fail);
%endif
            }
         '''

    mlnscript = '''
RESP "${y}"
IDEN 2 "${L2ID}"
IDEN 1 "_id"
% for i in range(len(x)):
ADDT "${x[i]}"
SETV 1 "${x[i]}"
% endfor
% for i in range(len(x2)):
SETV 2 "${x2[i]}"
% endfor
<%
def combs(s):
    out = []
    for i in range(0, len(s)):
        for j in range(0, i + 1):
            out.append(s[i] + '*' + s[j])
    return out

def combs1(a,x):
    out = []
    count=0
    for i in range(0, len(x)):
        for j in range(0, i + 1):
            if a[count]==False:
                out.append(x[i])
                out.append(x[j])
            count+=1
    return out

import numpy
complexvars = combs(x)
T=[]
for i in range(0, len(complexvars)):
    T.append(bool(context['inc' + str(i)]))

TT=combs1(T,x)
TT1=[]
for i in range(0,len(TT),2):
    TT1.append(TT[i])
TT2=[]   
for i in range(1,len(TT),2):
    TT2.append(TT[i])

%>

% for i in range(len(TT1)):
CLRE 1 "${TT1[i]}" "${TT2[i]}"
% endfor

<% M=len(x2)%>
% if M>1:
%if priors == 'Wishart':
<%
import numpy
TT = ' '.join([str(i) for i in numpy.zeros(len(x),dtype='int')])
TR = ' '.join(R.name)
%>
JOIN ${TT} 1 ${TR} ${str(v)} 0 c1092
PRIOR  c1092
% endif
% endif
'''

    latexMLwiN = r'''
<%
def combs(s):
    out = []
    for i in range(0, len(s)):
        for j in range(0, i + 1):
            out.append(s[i] + '*' + s[j])
    return out

def combs1(a,x):
    out = []
    count=0
    for i in range(0, len(x)):
        for j in range(0, i + 1):
            if a[count]==False:
                out.append(x[i])
                out.append(x[j])
            count+=1
    return out

import numpy
complexvars = combs(x)
T=[]
for i in range(0, len(complexvars)):
    T.append(bool(context['inc' + str(i)]))

TT=combs1(T,x)
TT1=[]
for i in range(0,len(TT),2):
    TT1.append(TT[i])
TT2=[]   
for i in range(1,len(TT),2):
    TT2.append(TT[i])

%>

\begin{aligned}
&\text{Select }\bold{Open}\text{ }\bold{Sample}\text{ } \bold{Worksheet} \text{ or }\bold{Open}\text{ } \bold{Worksheet} \text{ from the }\bold{File}\text{ menu.} \\ 
&\text{Select }\bold{datafile.dta}.\\ 
&\text{Select }\bold{Equations}\text{ from the }\bold{Model}\text{ menu.}\\ 
&\text{Click on } \mathbf{y}.\\ 
&\text{In the } \mathbf{y}\text{ list, select }\bold{${y}}\text{.}\\ 
&\text{In the }\bold{N}\text{ }\bold{levels}\text{ list, select } \mathbf{2}-\mathbf{ij}.\\ 
&\text{In the }\bold{level} \text{ }\mathbf{2(j):}\text{ list, select } \bold{${L2ID}}.\\  
&\text{In the }\bold{level} \text{ }\mathbf{1(i):}\text{ list, select } \bold{\_id}.\\  
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% for p in range(len(x)):
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{${x[p]}}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
&\text{In the }\bold{Equations}\text{ window click on }\bold{${x[p]}}.\\ 
&\text{Add the tick in the }(\mathbf{i})\bold{\_id}\text{ tick box.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% endfor
% for i in range(len(x2)):
&\text{In the }\bold{Equations}\text{ window click on }\bold{${x2[i]}}.\\ 
&\text{Add the tick in the }(\mathbf{j})\bold{${L2ID}}\text{ tick box.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% endfor

% for i in range(len(TT1)):
&\text{Click on the position for }\bold{${TT1[i]}}/\bold{${TT2[i]}}\text{ in the level 1 variance matrix}.\\ 
&\text{A message asking if you want to remove }\bold{${TT1[i]}}/\bold{${TT2[i]}}\text{ should appears}.\\ 
&\text{Choose }\bold{yes}\text{ to remove this term}.\\ 
% endfor

'''

    latexMLwiNextra = r'''
<% M=len(x2)%>
% if M>1:
&\text{Select the }\bold{MCMC}\text{ tab from the }\bold{Estimation}\text{ }\bold{control}\text{ window.}\\ 
&\text{Select }\bold{MCMC}/\bold{Priors}\text{ from the }\bold{Model}\text{ menu.}\\ 
&\text{Select }\bold{Gamma}\text{ }\bold{prior}\text{ from the } \bold{MCMC}\text{ }\bold{Priors}\text{ window (this sets the prior for level 1 variance).}\\ 
% if priors == 'Wishart':
&\text{Select }\bold{Informative}\text{ }\bold{prior} \text{ from the } \bold{MCMC}\text{ }\bold{Priors}\text{ window.}\\ 
&\text{Select }\Sigma_u \text{ from the top bar of the window.}\\ 
&\text{Input the estimates }
%for i in range(len(R.name)):
%if i==(len(R.name)-1):
\text{and }\mathbf{${R.name[i]}}
%else:
\mathbf{${R.name[i]}}\text{, }
%endif
%endfor
\text{ in the estimate boxes.}\\ 
&\text{Input } \mathbf{${v.name}} \text{ in the sample size box.}\\ 
&\text{Click on }\Sigma_u \text{ again.}\\ 
%endif
&\text{Click on the }\bold{Done}\text{ button.}\\ 
%endif
'''
