# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateNLevelBlock(Template):
    'Fits Normal response models allowing one or more higher classifications (nested or crossed) using a multivariate step for the fixed effects.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal' ]
    engines = ['eStat', 'WinBUGS', 'OpenBUGS', 'MLwiN_MCMC']

    inputs = '''
NumLevs = Integer('Number of Classifications: ')
for i in range(0, int(NumLevs)):
    selstr = 'Classification ' + str(i + 1) + ': '
    context['C' + str(i + 1)] = IDVector(selstr)

y = DataVector('Response: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)
storeresid = Boolean('Store residuals?')

beta = ParamVector(parents=[x], as_scalar=True, customstep=True)
tau = ParamScalar()
sigma2 = ParamScalar(modelled = False)

for i in range(0, int(NumLevs)):
    if storeresid:
        context['u' + str(i + 1)] = ParamVector(parents=[context['C' + str(i + 1)]], as_scalar=False)
    else:
        context['u' + str(i + 1)] = ParamVector(parents=[context['C' + str(i + 1)]], as_scalar=False, monitor=False)
    context['tau_u' + str(i + 1)] = ParamScalar()
    context['sigma2_u' + str(i + 1)] = ParamScalar(modelled = False)

deviance = ParamScalar(modelled = False)
'''

    model = '''
<% numlevs = int(NumLevs) %>

model {
    for (i in 1:length(${y})) {
        ${y}[i] ~ dnorm(mu[i], tau)
        mu[i] <- ${mmult(x, 'beta', 'i')} \\
        % for i in range(0, numlevs):
+ u${i + 1}[${context['C' + str(i + 1)]}[i]] \\
        % endfor

    }
    % for i in range(0, numlevs):
    for (i${i + 1} in 1:length(u${i + 1})) {
        u${i + 1}[i${i + 1}] ~ dnorm(0, tau_u${i + 1})
    }
    % endfor
    
    # Priors
    % for i in range(0, x.ncols()):
    beta_${i} ~ dflat()
    % endfor
    tau ~ dgamma(0.001000, 0.001000)
    sigma2 <- 1 / tau
    % for i in range(0, numlevs):
    tau_u${i + 1} ~ dgamma(0.001000, 0.001000)
    sigma2_u${i + 1} <- 1 / tau_u${i + 1}
    % endfor
}    
'''
   
    latex = r'''
<% numlevs = int(NumLevs) %>
\begin{aligned} 
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
${mmulttex(x, r'\beta', 'i')} 
% for i in range(0, numlevs):
+ u^{(${i + 2})}_{\mbox{${context['C' + str(i + 1)]}}[i]}
%endfor
\\ 
% for i in range(0, numlevs):
u^{(${i + 2})}_{\mbox{${context['C' + str(i + 1)]}}[i]} & \sim \mbox{N}(0,\sigma^2_{u${i + 1}}) \\ 
%endfor
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau \\ 
% for i in range(0, numlevs):
\tau_{u${i + 1}} & \sim \Gamma (0.001,0.001) \\ 
\sigma_{u${i + 1}}^2 & = 1 / \tau_{u${i + 1}} \\ 
% endfor
\end{aligned} 
'''

    preccode = '''
    {
<% nbeta = len(x) %> \\
<% numlevs = int(NumLevs) %> \\
        bool fail = false;
        static RectMatrix xtxchol(${nbeta},${nbeta});

        // Setting up constant terms for beta step
        if (runstate == 0) {
            xtxchol = matrix_cholesky(mat_x.T() * mat_x, false, fail);
        }

        // Multivariate step for beta
        RectMatrix ydiff(mat_y);
        for (int i = 0; i < ydiff.rows(); i++) {
% for k in range(0, numlevs):
            ydiff(i, 0) -= double(u${k + 1}[int(${context['C' + str(k + 1)]}[i])]);
% endfor
        }

        RectMatrix xty = mat_x.T() * ydiff;
        RectMatrix mean = matrix_cholsolve(xtxchol, xty);
        DiagMatrix taudiag = identity(${nbeta}) / tau;
        SymMatrix variance = matrix_cholsolve(xtxchol, taudiag);
        mat_beta = dmultnormal(mean, variance, fail);       
    }
'''

    mlnscript = '''
% for i in range(NumLevs):
NOTE CALC "${context['C' + str(i + 1)]}" = "${context['C' + str(i + 1)]}" + 1
% endfor
RESP "${y}"
% for i in range(NumLevs):
IDEN ${i + 2} "${context['C' + str(i + 1)]}"
% endfor
IDEN 1 "_id"
% if D == 'Normal':
ADDT "_levres"
% for i in range(NumLevs + 1):
SETV ${i + 1} "_levres"
% endfor
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% else:
print 'Distribution not recognised'
% endif
XCLA 1
'''
