# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateNLevModCorrClass(Template):
    'Fits random intercept models, allowing higher classifications (nested or crossed), two of which are specified as correlated. Allows for Normal, Poisson or binomial responses, and choice of uniform or Wishart priors.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal', 'Poisson', 'Binomial', 'Probit', 'Logit', 'Complementary log-log', 'Correlated classifications' ]
    engines = ['eStat']

    inputs = '''
NumLevs = Integer('Number of Classifications: ')
for i in range(0, int(NumLevs)):
    selstr = 'Classification ' + str(i + 1) + ': '
    context['C' + str(i + 1)] = IDVector(selstr)

CC1 = IDVector('First Correlated Classification: ')
CC2 = IDVector('Second Correlated Classification: ')
y = DataVector('Response: ')
D = Text('Specify distribution: ', ['Normal', 'Binomial', 'Poisson'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    link = Text('Specify link function: ', ['logit', 'probit', 'cloglog'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)

storeresid = Boolean('Store residuals?')

beta = ParamVector(parents=[x], as_scalar=True)

if D == 'Normal':
    tau = ParamScalar()
    sigma = ParamScalar(modelled = False)

omega = ParamMatrix(modelled = False, customstep=True)
omega.size = 2
d = ParamMatrix(customstep=True)
d.size = 2
priors = Text('Priors: ', ['Uniform', 'Wishart'])
if priors == 'Wishart':
    R = List('R matrix: ')
    v = Integer('Degrees of Freedom:')

for i in range(0, int(NumLevs)):
    if storeresid:
        context['u' + str(i + 1)] = ParamVector(parents=[context['C' + str(i + 1)]], as_scalar=False)
    else:
        context['u' + str(i + 1)] = ParamVector(parents=[context['C' + str(i + 1)]], as_scalar=False, monitor=False)
    context['tau_u' + str(i + 1)] = ParamScalar()
    context['sigma_u' + str(i + 1)] = ParamScalar(modelled = False)

if storeresid:
    v1 = ParamVector(parents=[CC1], as_scalar=False)
    v2 = ParamVector(parents=[CC2], as_scalar=False)
else:
    v1 = ParamVector(parents=[CC1], as_scalar=False, monitor=False)
    v2 = ParamVector(parents=[CC2], as_scalar=False, monitor=False)

deviance = ParamScalar(modelled = False)
'''

    model = '''
<% numlevs = int(NumLevs) %>

model {
    for (i in 1:length(${y})) {
        ${y}[i] ~
        % if D == 'Normal':
        dnorm(mu[i], tau)
        mu[i] <-
        % endif
        % if D == 'Binomial':
        dbin(p[i], ${n}[i])
        ${link}(p[i]) <-
        % endif

        % if D == 'Poisson':
        dpois(p[i])
        ${link}(p[i]) <- 
        % if offset:
        ${n}[i] +
        % endif
        % endif     
        ${mmult(x, 'beta', 'i')}
        % for i in range(0, numlevs):
        + u${i + 1}[${context['C' + str(i + 1)]}[i]]
        % endfor
        + v1[${CC1}[i]] + v2[${CC2}[i]]
    }
    % for i in range(0, numlevs):
    for (i${i + 1} in 1:length(u${i + 1})) {
        u${i + 1}[i${i + 1}] ~ dnorm(0, tau_u${i + 1})
    }
    % endfor

    for (j in 1:length(v1)) {
        dummy[j] ~ dnormal2a(v1[j], v2[j], 0, 0, d[0], d[1], d[2])
        dum[j] ~ ddummy(dummy[j])
        v1[j] ~ dflat()
        v2[j] ~ dflat()
    }    

    # Priors
    % for i in range(0, x.ncols()):
    beta_${i} ~ dflat()
    % endfor
    % if D == 'Normal':
    tau ~ dgamma(0.001000, 0.001000)
    sigma <- 1 / sqrt(tau)
    % endif
    % for i in range(0, numlevs):
    tau_u${i + 1} ~ dgamma(0.001000, 0.001000)
    sigma_u${i + 1} <- 1 / sqrt(tau_u${i + 1})
    % endfor
}    
'''
   
    latex = r'''
<% numlevs = int(NumLevs) %>
\begin{aligned} 
%if D == 'Normal':
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
%if D == 'Binomial':
 \mbox{${y}}_{i} & \sim \mbox{Binomial}(\mbox{${n}}_{i}, \pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%endif
%if D == 'Poisson':
 \mbox{${y}}_{i} & \sim \mbox{Poisson}(\pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%if offset:
\mbox{${n}}_{i} +
%endif
%endif
%if D == 't' :
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
${mmulttex(x, r'\beta', 'i')} 
% for i in range(0, numlevs):
+ u^{(${i + 2})}_{\mbox{${context['C' + str(i + 1)]}}[i]} 
%endfor
+ v^{(1)}_{\mbox{${CC1}}[i]} + v^{(2)}_{\mbox{${CC2}}[i]}
\\ 
%if D=='Normal':
% for i in range(0, numlevs):
u^{(${i + 2})}_{\mbox{${context['C' + str(i + 1)]}}[i]} & \sim \mbox{N}(0,\sigma^2_{u${i + 1}}) \\ 
%endfor
%endif
%if D=='t':
% for i in range(0, numlevs):
u^{(${i + 2})}_{\mbox{${context['C' + str(i + 1)]}}[i]} & \sim \mbox{t}(0,\sigma^2_{u${i + 1}}, \df) \\ 
%endfor
%endif
\left(
\begin{array}{l}
v^{(1)}_{\mbox{${CC1}}[i]}  \\ 
v^{(2)}_{\mbox{${CC2}}[i]} 
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
0 \\ 
0
\end{array}
\right), \Omega_{v} \right] \\ 
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
%if D == 'Normal':
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau \\ 
%endif
% for i in range(0, numlevs):
\tau_{u${i + 1}} & \sim \Gamma (0.001,0.001) \\ 
\sigma_{u${i + 1}}^2 & = 1 / \tau_{u${i + 1}} \\ 
% endfor
% if context['priors'] == 'Wishart':
\Omega_v & \sim \mbox{inverse Wishart}_{2}[${context['v']}*R_v,${context['v']}],R_v =
\left[\begin{array}{
ll
}
<% lenR = 0%>
% for i in range(0, 2):
% for j in range(0, 2):
% if j <= i:
${context['R'].name[lenR]}
<% lenR += 1 %>
% endif
% if j != 1:
&
% else:
% if i != 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if context['priors'] == 'Uniform':
\Omega_v & \propto 1
% endif
\\ 
%if D=='t':
\df & = 8
%endif
\end{aligned} 
'''
   
    preccode = '''
    {
        // Note currently using a uniform prior for variance matrix
        SymMatrix sb(2);
        for(int i = 0; i < length(v1); i++) {
% for j in range(0, 2):
% for k in range(j, 2):
            sb(${j}, ${k}) += double(v${j + 1}[i]) * double(v${k + 1}[i]);
% endfor
% endfor
        }
% if priors == 'Uniform':
        int vw = length(v1)-3;
        if (runstate == 0) {
% for j in range(0, 2):
            sb(${j}, ${j}) += 0.0001;
% endfor
        }
% endif
% if priors == 'Wishart':
        int vw = length(v1) + ${v};
% endif
% if priors == 'Wishart':
<%
import numpy
Rmat = numpy.empty([2, 2])
count = 0 
for j in range(0, 2):
    for k in range(0, j + 1):
        Rmat[j, k] = float(R.name[count]) 
        Rmat[k, j] = Rmat[j, k]
        count += 1
%>

<% count = 0 %>
% for j in range(0, 2):
% for k in range(j, 2):
        sb(${j}, ${k}) += ${str(Rmat[j, k] * float(v))};
% endfor
% endfor
% endif
        matrix_sym_invinplace(sb);
        bool fail = false;
        mat_d = dwishart(vw, sb, fail);
        mat_omega = matrix_sym_inverse(mat_d, fail);
    }
'''
