# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateNLevXCHC(Template):
    'Fits random intercept models allowing one or more higher classifications (nested or crossed), using hierarchical centring of fixed effects. Allows for Normal, Poisson or binomial responses.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal', 'Poisson', 'Binomial', 'Probit', 'Logit', 'Complementary log-log', 'Alternative MCMC methods' ]
    engines = ['eStat', 'WinBUGS', 'OpenBUGS', 'JAGS']

    inputs = '''
NumClass = Integer('Number of Classifications (including level 1): ')
for i in range(1, int(NumClass)):
    selstr = 'Classification ' + str(i+1) + ': '
    context['C' + str(i)] = IDVector(selstr)

y = DataVector('Response: ')
D = Text('Specify distribution: ', ['Normal', 'Binomial', 'Poisson'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    denom = DataVector(value = 'denom')
    link = Text('Specify link function: ', ['logit', 'probit', 'cloglog'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')

x = DataMatrix('Explanatory variables at observation level: ', allow_cat = True)
for lev in range(1, int(NumClass)):
    context['L'+str(lev)+'Data'] = DataFileChooser('Classification ' + str(lev + 1)+' data set') 
    context['x'+str(lev)] = DataMatrix('Explanatory variables centred at ' + str(context['C' + str(lev)]) + ' classification: ', data_src = context['L'+str(lev)+'Data'], allow_cat = True)

storeresid = Boolean('Store residuals?')

beta = ParamVector(as_scalar=True)
beta.ncols = len(x)
for lev in range(1, int(NumClass)):
    beta.ncols += len(context['x'+str(lev)])

if D == 'Normal':
    tau = ParamScalar()
    sigma2 = ParamScalar(modelled = False)

for i in range(1, int(NumClass)):
    context['tau_u' + str(i)] = ParamScalar()
    context['sigma2_u' + str(i)] = ParamScalar(modelled = False)
    if storeresid:
        context['u' + str(i)] = ParamVector(parents=[context['C' + str(i)]], as_scalar=False)
    else:
        context['u' + str(i)] = ParamVector(parents=[context['C' + str(i)]], as_scalar=False, monitor=False)

deviance = ParamScalar(modelled = False)
'''

    model = '''
<% numlevs = int(NumClass) %>

model {
        for (i in 1:length(${y})) {
            ${y}[i] ~ \\
            % if D == 'Normal':
dnorm(mu[i], tau)
            mu[i] <- \\
            % endif
            % if D == 'Binomial':
dbin(p[i], denom[i])
            ${link}(p[i]) <- \\
            % endif
            % if D == 'Poisson':
dpois(p[i])
            ${link}(p[i]) <- \\
            % if offset:
${n}[i] + \\
            % endif
            % endif  
<% first = True %>\\
<% b = 0 %>
            % for j in range(0,len(x)):
            % if first == False:
+ \\
            % endif  
<% first = False %>\\
${x[j]}[i] * beta_${b} \\
<% b += 1 %> \\
            % endfor
            % for i in range(1, numlevs):
+ u${i}[${context['C' + str(i)]}[i]] \\
            % endfor

        }
    % for i in range(1, numlevs):
    for (i${i} in 1:length(u${i})) {
        u${i}[i${i}] ~ dnorm(mu.u${i}[i${i}], tau_u${i})
        mu.u${i}[i${i}] <- \\
<% first = True %>\\
        % for j in range(0,len(context['x'+str(i)])):
        % if first == False:
+ \\
        % endif
<% first = False %>\\
${context['L'+str(i)+'Data']}_${context['x'+str(i)][j]}[i${i}] * beta_${b} \\
<% b += 1 %> \\
        % endfor
% if first == True:
0
% endif
    }
    % endfor
    
    # Priors
    % for i in range(0, beta.ncols):
    beta_${i} ~ dflat()
    % endfor
    % if D == 'Normal':
    tau ~ dgamma(0.001000, 0.001000)
    sigma2 <- 1 / tau
    % endif
    % for i in range(1, numlevs):
    tau_u${i} ~ dgamma(0.001000, 0.001000)
    sigma2_u${i} <- 1 / tau_u${i}
    % endfor
}    
'''
       
    latex = r'''
<% numlevs = int(NumClass) %>
\begin{aligned} 
%if D == 'Normal':
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
%if D == 'Binomial':
 \mbox{${y}}_{i} & \sim \mbox{Binomial}(\mbox{${n}}_{i}, \pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%endif
%if D == 'Poisson':
 \mbox{${y}}_{i} & \sim \mbox{Poisson}(\pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%if offset:
\mbox{${n}}_{i} +
%endif
%endif
<% first = True %>
<% b = 0 %>
% for j in range(0,len(x)):
% if first == False:
+
% endif  
<% first = False %>
\mbox{${x[j]}}_i * \beta_${b}
<% b += 1 %>
% endfor
% for i in range(1, numlevs):
+ u^{(${i + 1})}_{\mbox{${context['C' + str(i)]}}[i]}
% endfor
\\ 
% for i in range(1, numlevs):
u^{(${i + 1})}_j & \sim \mbox{N}(\mu^{(${i + 1})}_j,\sigma^2_{u${i + 1}}) \\ 
\mu^{(${i + 1})}_j & =
<% first = True %>
% for j in range(0,len(context['x'+str(i)])):
% if first == False:
+ 
% endif
<% first = False %>
\mbox{${context['x'+str(i)][j]}}_j * \beta_${b}
<% b += 1 %>
% endfor
% if first == True:
0
% endif
\\ 
%endfor
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
%if D == 'Normal':
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau \\ 
%endif
% for i in range(1, numlevs):
\tau_{u${i + 1}} & \sim \Gamma (0.001,0.001) \\ 
\sigma_{u${i + 1}}^2 & = 1 / \tau_{u${i + 1}} \\ 
% endfor
\end{aligned} 
'''
        
    preparedata = '''
mydata = data['datafile']
numlevs = int(NumClass)

if D == 'Binomial':
    mydata.addvariable('denom', data = mydata.variables[n]['data'])
'''
