# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateNLevXCHCMLwiN(Template):
    'Fits random intercept models allowing one or more higher classifications (nested or crossed), using hierarchical centring of fixed effects. Allows for Normal, Poisson or binomial responses.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal', 'Poisson', 'Binomial', 'Probit', 'Logit', 'Complementary log-log', 'Alternative MCMC methods' ]
    engines = ['MLwiN_MCMC']

    inputs = '''
NumClass = Integer('Number of Classifications (including level 1): ')
NumLevs = Integer(value=int(NumClass) -1)
for i in range(1, int(NumClass)):
    selstr = 'Classification ' + str(i+1) + ': '
    context['C' + str(i)] = IDVector(selstr)
    context['tau_u' + str(i)] = ParamScalar()
    context['sigma2_u' + str(i)] = ParamScalar(modelled = False)
    context['u' + str(i)] = ParamVector(parents=[context['C' + str(i)]], as_scalar=False)
y = DataVector('Response: ')
D = Text('Specify distribution: ', ['Normal', 'Binomial', 'Poisson'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    denom = DataVector(value = 'denom')
    link = Text('Specify link function: ', ['logit', 'probit', 'cloglog'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
if D == 'Normal':
    tau = ParamScalar()
    sigma2 = ParamScalar(modelled = False)
x = DataMatrix('Explanatory variables: ', allow_cat = True)
for i in x:
    context['lev' + i] = Integer('Classification for ' + i + ': ')
beta = ParamVector(parents=[x], as_scalar=True)
'''
      
    latex = r'''
<% numlevs = int(NumClass) %>
\begin{aligned} 
%if D == 'Normal':
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
%if D == 'Binomial':
 \mbox{${y}}_{i} & \sim \mbox{Binomial}(\mbox{${n}}_{i}, \pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%endif
%if D == 'Poisson':
 \mbox{${y}}_{i} & \sim \mbox{Poisson}(\pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%if offset:
\mbox{${n}}_{i} +
%endif
%endif
<% first = True %>
% for j in range(0,len(x)):
% if int(context['lev' + x[j]]) == 1:
% if first == False:
+
% endif  
<% first = False %>             
\mbox{${x[j]}}_i * \beta_${j}
% endif
% endfor
% for i in range(1, numlevs):
+ u^{(${i + 1})}_{\mbox{${context['C' + str(i)]}}[i]}
%endfor
\\ 
% for i in range(1, numlevs):
u^{(${i + 1})}_j & \sim \mbox{N}(\mu^{(${i + 1})}_j,\sigma^2_{u${i + 1}}) \\ 
\mu^{(${i + 1})}_j & =
<% first = True %>
% for j in range(0,len(x)):
% if int(context['lev' + x[j]]) == (i + 1):
% if first == False:
+ 
% endif
<% first = False %>
\mbox{${x[j]}}_j * \beta_${j}
% endif
% endfor
% if first == True:
0
% endif
\\ 
%endfor
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
%if D == 'Normal':
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau \\ 
%endif
% for i in range(1, numlevs):
\tau_{u${i + 1}} & \sim \Gamma (0.001,0.001) \\ 
\sigma_{u${i + 1}}^2 & = 1 / \tau_{u${i + 1}} \\ 
% endfor
\end{aligned} 
'''

    mlnscript = '''
% for i in range(NumLevs):
NOTE CALC "${context['C' + str(i + 1)]}" = "${context['C' + str(i + 1)]}" + 1
% endfor
RESP "${y}"
% for i in range(NumLevs):
IDEN ${i + 2} "${context['C' + str(i + 1)]}"
% endfor
IDEN 1 "_id"
% if D == 'Binomial':
RDISt 1 0
DOFFs 1 "${n}"
% if link == 'logit':
LFUN 0
% elif link == 'probit':
LFUN 1
% elif link == 'cloglog':
LFUN 2
% endif
ADDT "_levres"
% for i in range(1, NumLevs + 1):
SETV ${i + 1} "_levres"
% endfor
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% elif D == 'Normal':
ADDT "_levres"
% for i in range(NumLevs + 1):
SETV ${i + 1} "_levres"
% endfor
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% elif D == 'Poisson':
RDISt 1 1
LFUN 3
% if offset:
DOFFs 1 ${n}
% endif
ADDT "_levres"
% for i in range(1, NumLevs + 1):
SETV ${i + 1} "_levres"
% endfor
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% endif
XCLA 1

<% highlev = 2 %>
% for i in x:
% if int(context['lev' + i]) > highlev:
<% highlev = int(context['lev' + i]) %>
% endif
% endfor

HCEN 1 ${highlev}
'''

    preparedata = '''
mydata = data['datafile']
numlevs = int(NumClass)

if D == 'Binomial':
    mydata.addvariable('denom', data = mydata.variables[n]['data'])
'''
